<?php
/**
 * Install class
 */
namespace Multivendor\Addon\Install;

if ( ! class_exists( __NAMESPACE__ . '\\Install' ) ) {
	/**
	 * Install class
	 */
	class Install {
		/**
		 * Repository
		 *
		 * @var string
		 */
		protected string $repository;

		/**
		 * Access token
		 *
		 * @var string
		 */
		protected string $access_token;


		/**
		 * Plugin slug
		 *
		 * @var string
		 */
		protected string $plugin_slug;

		/**
		 * Plugin
		 *
		 * @var string
		 */
		protected string $plugin;

		/**
		 * Init Installr
		 *
		 * @param  array $args Arguments.
		 * @return void
		 */
		public static function init( array $args ): void {
			$instance              = new self();
			$instance->plugin_slug = 'multivendor-addon-for-jitsi-meet';
			$instance->plugin      = 'multivendor-addon-for-jitsi-meet/jitsi-meet-wp.php';

			$instance->repository   = isset( $args['repository'] ) ? sanitize_text_field( $args['repository'] ) : '';
			$instance->access_token = isset( $args['access_token'] ) ? sanitize_text_field( $args['access_token'] ) : '';

			if ( ! empty( $instance->repository ) ) {
				$instance->run_installer();
			}
		}

		/**
		 * Plugin Installr
		 *
		 * @return void
		 */
		public function run_installer(): void {
			add_filter( 'http_request_args', array( $this, 'set_request_args' ), 10, 2 );
			add_filter( 'upgrader_source_selection', array( $this, 'set_upgrader_source_selection' ), 11, 4 );
			$this->install_plugin();
		}

		public function install_plugin() {
			include_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
			wp_cache_flush();

			$response = $this->get_repo_info();

			$upgrader  = new \Plugin_Upgrader();
			$installed = $upgrader->install( $response['package'] );

			return $installed;
		}

		/**
		 * Get headers
		 *
		 * @return array
		 */
		public function get_headers(): array {
			$headers = array(
				'User-Agent' => 'WP Plugin Installr',
			);

			if ( ! empty( $this->access_token ) ) {
				$headers['Authorization'] = 'Bearer ' . $this->access_token;
			}

			return $headers;
		}

		/**
		 * Get repository info
		 *
		 * @param  object $transient Transient.
		 * @throws \Exception Invalid response.
		 * @return object|null
		 */
		public function get_repo_info( $transient = null ) {

			$url = wp_sprintf( 'https://api.github.com/repos/%s/releases/latest', esc_attr( $this->repository ) );

			$response = wp_remote_get( $url, array( 'headers' => $this->get_headers() ) );

			// Bail if response is not valid.
			if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {
				return;
			}

			$response = json_decode( wp_remote_retrieve_body( $response ) );

			if ( empty( $response->tag_name ) ) {
				return;
			}

			$install_info = array(
				'slug'        => $this->plugin_slug,
				'new_version' => $response->tag_name,
				'package'     => $response->zipball_url,
			);

			set_transient( 'jitsi_meet_addon_install_info', $install_info, 60 );

			return $install_info;
		}

		/**
		 * Set request args
		 *
		 * @param  array  $args Args.
		 * @param  string $url  URL.
		 * @return array
		 */
		public function set_request_args( array $args, string $url ): array {
			$install_info = get_transient( 'jitsi_meet_addon_install_info' );

			// new version.
			if ( empty( $install_info['new_version'] ) ) {
				return $args;
			}

			$expected_url = wp_sprintf( 'https://api.github.com/repos/%s/zipball/%s', esc_attr( $this->repository ), esc_attr( $install_info['new_version'] ) );

			if ( $url !== $expected_url ) {
				return $args;
			}

			// set headers.
			$args['headers'] = $this->get_headers();

			return $args;
		}

		/**
		 * Set upgrader source selection
		 *
		 * Zip from Github contains a folder with the name of the commit hash. So we need to rename it to the plugin slug.
		 *
		 * @param  string $source        Source.
		 * @param  string $remote_source Remote source.
		 * @param  object $upgrader      Upgrader.
		 * @param  array  $hook_extra    Hook extra.
		 * @return string
		 */
		public function set_upgrader_source_selection( string $source, string $remote_source, object $upgrader, array $hook_extra ): string {

			// rename source path to plugin slug.
			$new_source = $remote_source . '/' . $this->plugin_slug;

			// rename source path to plugin slug.
			rename( $source, $new_source );

			return $new_source;
		}
	}
}
